precision highp float;
uniform sampler2D u_Texture;
varying vec2 v_TexCoordOut;
uniform int is_fisheye;
uniform float distortion;
uniform float space;

highp vec3 fisheye(highp vec2 vec2xy) {
    lowp vec2 distortion_center = vec2(0.5, 0.5);

    highp float distortion_x;
    highp float distortion_y;
    highp float rr;
    highp float r2;
    highp float theta;
    // 变形程度 -0.8 -> 1.5
    highp float distortion_k1 = mix(-0.15, 0.5, distortion);
    highp float distortion_k2 = 0.5;
    highp float scale = mix(1.02, 0.88, distortion);

    rr = sqrt((vec2xy.x - distortion_center.x) * (vec2xy.x - distortion_center.x) + (vec2xy.y - distortion_center.y) * (vec2xy.y - distortion_center.y));
    r2 = rr * (1.0 + distortion_k1 * (rr * rr) + distortion_k2 * (rr * rr * rr * rr));
    theta = atan(vec2xy.x - distortion_center.x, vec2xy.y - distortion_center.y);
    distortion_x = sin(theta) * r2 * scale;
    distortion_y = cos(theta) * r2 * scale;
    highp float dx = distortion_x + 0.5;
    highp float dy = distortion_y + 0.5;
    lowp float b = 0.1;
    if(dx < 0.0 || dx > 1.0 || dy < 0.0 || dy > 1.0) {
        b = -0.1;
    }
    return vec3(dx, dy, b);
}

void main() {
    float scale = 1.0 + space;
    vec2 loc = v_TexCoordOut;
    vec4 color = vec4(0.0, 0.0, 0.0, 1.0);
    loc.x *= scale;
    loc.y = (loc.y * scale) - space * 0.5;
    int black = 0;
    if(v_TexCoordOut.x < 0.5) {
        if(loc.x > 0.5) {
            black = 1;
        }
    } else {
        loc.x -= space;
        if(loc.x < 0.5) {
            black = 1;
        }
    }
    if(loc.y < 0.0 || loc.y > 1.0 || black == 1) {

    } else {
        if(is_fisheye == 1) {
            float tx;
            float ty = loc.y;
            if(loc.x < 0.5) {
                tx = loc.x * 2.0;
            } else {
                tx = loc.x * 2.0 - 1.0;
            }
            vec3 loc_f = fisheye(vec2(tx, ty));
            if(loc_f.z > 0.0) {
                loc_f.x *= 0.5;
                if(loc.x > 0.5) {
                    loc_f.x += 0.5;
                }
                loc = loc_f.xy;
            } else {
                black = 1;
            }
        }
        if(black == 0) {
            color = texture2D(u_Texture, loc);
        }
    }

    gl_FragColor = color;
}
